# Generated by Django 3.1.14 on 2024-08-19 14:23

from django.db import (
    migrations,
    models,
)


class Migration(migrations.Migration):
    dependencies = [
        ('edu_rdm_integration', '0009_auto_20240522_1619'),
    ]

    operations = [
        migrations.AddField(
            model_name='transferredentity',
            name='export_enabled',
            field=models.BooleanField(default=True, verbose_name='Включение экспорта для сущности'),
        ),
    ]
