# Generated by Django 3.2.24 on 2025-07-04 08:10

from django.db import (
    migrations,
)


class Migration(migrations.Migration):
    dependencies = [
        ('edu_rdm_integration_export_data_stage', '0001_initial'),
        ('edu_rdm_integration_transfer_pipeline', '0001_initial'),
        ('edu_rdm_integration_entities', '0001_initial'),
    ]

    operations = [
        migrations.RenameModel(
            old_name='RegionalDataMartEntityEnum',
            new_name='RDMEntityEnum',
        ),
    ]
