# Generated by Django 3.2.24 on 2025-06-28 14:08

from django.db import (
    migrations,
    models,
)


class Migration(migrations.Migration):
    initial = True

    dependencies = [
        ('edu_rdm_integration', '0017_delete_uploaddatacommand'),
    ]

    operations = [
        migrations.SeparateDatabaseAndState(
            state_operations=[
                migrations.CreateModel(
                    name='RegionalDataMartModelEnum',
                    fields=[
                        ('title', models.TextField(verbose_name='расшифровка значения')),
                        (
                            'key',
                            models.CharField(
                                db_index=True, max_length=512, primary_key=True, serialize=False, verbose_name='ключ'
                            ),
                        ),
                        ('order_number', models.PositiveIntegerField(default=100000, verbose_name='Порядковый номер')),
                    ],
                    options={
                        'verbose_name': 'Модель-перечисление моделей "Региональной витрины данных"',
                        'verbose_name_plural': 'Модели-перечисления моделей "Региональной витрины данных"',
                        'db_table': 'rdm_model',
                    },
                ),
            ],
            database_operations=[],
        ),
    ]
