# Generated by Django 3.2.24 on 2025-07-04 08:10

from django.db import (
    migrations,
)


class Migration(migrations.Migration):
    dependencies = [
        ('function_tools', '0005_auto_20220724_0050'),
        ('edu_rdm_integration_collect_data_stage', '0002_edurdmcollectdatacommandprogress'),
    ]

    operations = [
        migrations.RenameModel(
            old_name='CollectingDataStageStatus',
            new_name='RDMCollectingDataStageStatus',
        ),
        migrations.RenameModel(
            old_name='CollectingExportedDataStage',
            new_name='RDMCollectingDataStage',
        ),
        migrations.RenameModel(
            old_name='CollectingDataSubStageStatus',
            new_name='RDMCollectingDataSubStageStatus',
        ),
        migrations.RenameModel(
            old_name='CollectingExportedDataSubStage',
            new_name='RDMCollectingDataSubStage',
        ),
        migrations.RenameModel(
            old_name='EduRdmCollectDataCommandProgress',
            new_name='RDMCollectingDataCommandProgress',
        ),
    ]
