# Generated by Django 3.2.24 on 2025-07-04 08:10

from django.db import (
    migrations,
)


class Migration(migrations.Migration):
    dependencies = [
        ('function_tools', '0005_auto_20220724_0050'),
        ('edu_rdm_integration_upload_data_stage', '0001_initial'),
        ('async_task', '0003_alter_runningtask_options'),
        ('edu_rdm_integration_entities', '0002_rename_regionaldatamartentityenum_rdmentityenum'),
        ('edu_rdm_integration_export_data_stage', '0001_initial'),
    ]

    operations = [
        migrations.RenameModel(
            old_name='EduRdmExportDataCommandProgress',
            new_name='RDMExportingDataCommandProgress',
        ),
        migrations.RenameModel(
            old_name='ExportingDataStage',
            new_name='RDMExportingDataStage',
        ),
        migrations.RenameModel(
            old_name='ExportingDataStageStatus',
            new_name='RDMExportingDataStageStatus',
        ),
        migrations.RenameModel(
            old_name='ExportingDataSubStage',
            new_name='RDMExportingDataSubStage',
        ),
        migrations.RenameModel(
            old_name='ExportingDataSubStageAttachment',
            new_name='RDMExportingDataSubStageAttachment',
        ),
        migrations.RenameModel(
            old_name='ExportingDataSubStageEntity',
            new_name='RDMExportingDataSubStageEntity',
        ),
        migrations.RenameModel(
            old_name='ExportingDataSubStageStatus',
            new_name='RDMExportingDataSubStageStatus',
        ),
    ]
