# Generated by Django 3.2.24 on 2025-06-28 14:22

from django.db import (
    migrations,
)


class Migration(migrations.Migration):
    initial = True

    dependencies = [
        ('uploader_client', '0001_initial'),
    ]

    operations = [
        migrations.SeparateDatabaseAndState(
            state_operations=[
                migrations.CreateModel(
                    name='UploaderClientLog',
                    fields=[],
                    options={
                        'proxy': True,
                        'indexes': [],
                        'constraints': [],
                    },
                    bases=('uploader_client.entry',),
                ),
            ],
            database_operations=[],
        ),
    ]
