# Generated by Django 3.2.23 on 2025-02-04 14:13

from django.db import (
    migrations,
    models,
)


class Migration(migrations.Migration):
    dependencies = [
        ('rdm_collect_and_export_data', '0001_initial'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='edurdmcollectdatacommandprogress',
            options={'verbose_name': 'Задача по сбору данных', 'verbose_name_plural': 'Задачи по сбору данных'},
        ),
        migrations.AlterModelOptions(
            name='edurdmexportdatacommandprogress',
            options={'verbose_name': 'Команда экспорта данных', 'verbose_name_plural': 'Команды экспорта данных'},
        ),
        migrations.AlterField(
            model_name='edurdmcollectdatacommandprogress',
            name='institute_ids',
            field=models.JSONField(blank=True, default=list, null=True, verbose_name='id учебного заведения'),
        ),
    ]
