# Generated by Django 3.2.24 on 2025-06-28 16:04

import django.db.models.deletion
from django.db import (
    migrations,
    models,
)


class Migration(migrations.Migration):
    initial = True

    dependencies = [
        ('edu_rdm_integration_entities', '0001_initial'),
    ]

    operations = [
        migrations.SeparateDatabaseAndState(
            state_operations=[
                migrations.CreateModel(
                    name='TransferredEntity',
                    fields=[
                        (
                            'id',
                            models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID'),
                        ),
                        (
                            'export_enabled',
                            models.BooleanField(default=True, verbose_name='Включение экспорта для сущности'),
                        ),
                        (
                            'queue_level',
                            models.PositiveIntegerField(
                                choices=[(1, 'Быстрая'), (2, 'Основная'), (3, 'Долгая')],
                                default=2,
                                verbose_name='Уровень очереди',
                            ),
                        ),
                        (
                            'entity',
                            models.OneToOneField(
                                on_delete=django.db.models.deletion.CASCADE,
                                to='edu_rdm_integration_entities.regionaldatamartentityenum',
                                verbose_name='Сущность',
                            ),
                        ),
                    ],
                    options={
                        'verbose_name': 'Сущность, по которой должен быть произведен сбор и экспорт данных',
                        'verbose_name_plural': 'Сущности, по которым должен быть произведен сбор и экспорт данных',
                        'db_table': 'rdm_transferred_entity',
                    },
                ),
            ],
            database_operations=[],
        ),
    ]
