# Generated by Django 3.2.24 on 2025-07-04 07:25

from django.db import (
    migrations,
)


class Migration(migrations.Migration):
    dependencies = [
        ('rdm_collect_and_export_data', '0002_auto_20250204_1413'),
    ]

    operations = [
        migrations.SeparateDatabaseAndState(
            state_operations=[
                migrations.RemoveField(
                    model_name='edurdmexportdatacommandprogress',
                    name='entity',
                ),
                migrations.RemoveField(
                    model_name='edurdmexportdatacommandprogress',
                    name='stage',
                ),
                migrations.RemoveField(
                    model_name='edurdmexportdatacommandprogress',
                    name='task',
                ),
                migrations.DeleteModel(
                    name='EduRdmCollectDataCommandProgress',
                ),
                migrations.DeleteModel(
                    name='EduRdmExportDataCommandProgress',
                ),
            ],
            database_operations=[],
        ),
    ]
