# Generated by Django 3.2.24 on 2025-10-08 14:17

import django.db.models.deletion
from django.db import (
    migrations,
    models,
)


class Migration(migrations.Migration):

    dependencies = [
        ('edu_rdm_integration_collect_data_stage', '0004_auto_20250704_0825'),
    ]

    operations = [
        migrations.AlterField(
            model_name='rdmcollectingdatasubstage',
            name='previous',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, to='edu_rdm_integration_collect_data_stage.rdmcollectingdatasubstage', verbose_name='Предыдущий сбор данных'),
        ),
    ]
